@@ -27,7 +27,7 @@
 #include "libofx.h"
 #include "ofx_containers.hh"
 
-OfxMainContainer::OfxMainContainer(LibofxContext *p_libofx_context, OfxGenericContainer *para_parentcontainer, string para_tag_identifier):
+OfxMainContainer::OfxMainContainer(LibofxContext *p_libofx_context, OfxGenericContainer *para_parentcontainer, std::string para_tag_identifier):
   OfxGenericContainer(p_libofx_context, para_parentcontainer, para_tag_identifier)
 {
 
@@ -59,7 +59,9 @@ int OfxMainContainer::add_container(OfxGenericContainer * container)
   message_out(DEBUG, "OfxMainContainer::add_container for element " + container->tag_identifier + "; destroying the generic container");
   /* Call gen_event anyway, it could be a status container or similar */
   container->gen_event();
-  delete container;
+  if (container != this) {
+    delete container;
+  }
   return 0;
 }
 
@@ -209,7 +211,7 @@ int  OfxMainContainer::gen_event()
   return true;
 }
 
-OfxSecurityData *  OfxMainContainer::find_security(string unique_id)
+OfxSecurityData *  OfxMainContainer::find_security(std::string unique_id)
 {
   message_out(DEBUG, "OfxMainContainer::find_security() Begin.");
 
@@ -219,7 +221,7 @@ OfxSecurityData *  OfxMainContainer::find_security(string unique_id)
   {
     if (((OfxSecurityContainer*)(*tmp))->data.unique_id == unique_id)
     {
-      message_out(DEBUG, (string)"Security " + ((OfxSecurityContainer*)(*tmp))->data.unique_id + " found.");
+      message_out(DEBUG, (std::string)"Security " + ((OfxSecurityContainer*)(*tmp))->data.unique_id + " found.");
       retval = &((OfxSecurityContainer*)(*tmp))->data;
     }
     ++tmp;
