@@ -33,11 +33,11 @@
 #include "ofx_containers.hh"
 #include "ofx_sgml.hh"
 
-using namespace std;
 
 OfxMainContainer * MainContainer = NULL;
 extern SGMLApplication::OpenEntityPtr entity_ptr;
 extern SGMLApplication::Position position;
+static const std::string MESSAGE_NON_SGML_CHAR = "non SGML character";
 
 
 /** \brief This object is driven by OpenSP as it parses the SGML from the ofx file(s)
@@ -48,8 +48,9 @@ private:
   OfxGenericContainer *curr_container_element; /**< The currently open object from ofx_proc_rs.cpp */
   OfxGenericContainer *tmp_container_element;
   bool is_data_element; /**< If the SGML element contains data, this flag is raised */
-  string incoming_data; /**< The raw data from the SGML data element */
+  std::string incoming_data; /**< The raw data from the SGML data element */
   LibofxContext * libofx_context;
+  unsigned errorCountToIgnore = 0;
 
 public:
 
@@ -65,14 +66,15 @@ public:
     message_out(DEBUG, "Entering the OFXApplication's destructor");
   }
 
+  unsigned getErrorCountToIgnore() const { return errorCountToIgnore; }
+
   /** \brief Callback: Start of an OFX element
    *
    An OpenSP callback, get's called when the opening tag of an OFX element appears in the file
   */
   void startElement (const StartElementEvent & event)
   {
-    string identifier;
-    CharStringtostring (event.gi, identifier);
+    std::string identifier = CharStringtostring (event.gi);
     message_out(PARSER, "startElement event received from OpenSP for element " + identifier);
 
     position = event.pos;
@@ -138,7 +140,7 @@ public:
       else if (identifier == "STMTTRN")
       {
         message_out (PARSER, "Element " + identifier + " found");
-        if (curr_container_element->type == "INVESTMENT")
+        if (curr_container_element && curr_container_element->type == "INVESTMENT")
         {
           //push up to the INVBANKTRAN OfxInvestmentTransactionContainer
           curr_container_element = new OfxPushUpContainer (libofx_context, curr_container_element, identifier);
@@ -198,10 +200,11 @@ public:
       {
         message_out (PARSER, "Element " + identifier + " found");
         /* check the container to avoid creating multiple statements for TRANSFERs */
-        if (curr_container_element->type == "STATEMENT"
+        if (curr_container_element && 
+            (  curr_container_element->type == "STATEMENT"
             || curr_container_element->tag_identifier == "BANKACCTINFO"
             || curr_container_element->tag_identifier == "CCACCTINFO"
-            || curr_container_element->tag_identifier == "INVACCTINFO")
+            || curr_container_element->tag_identifier == "INVACCTINFO"))
           curr_container_element = new OfxAccountContainer (libofx_context, curr_container_element, identifier);
         else
           // no new account or statement for a <TRANSFER>
@@ -267,11 +270,8 @@ public:
   */
   void endElement (const EndElementEvent & event)
   {
-    string identifier;
-    bool end_element_for_data_element;
-
-    CharStringtostring (event.gi, identifier);
-    end_element_for_data_element = is_data_element;
+    std::string identifier = CharStringtostring (event.gi);
+    bool end_element_for_data_element = is_data_element;
     message_out(PARSER, "endElement event received from OpenSP for element " + identifier);
 
     position = event.pos;
@@ -363,7 +363,7 @@ public:
   */
   void data (const DataEvent & event)
   {
-    string tmp;
+    std::string tmp;
     position = event.pos;
     AppendCharStringtostring (event.data, incoming_data);
     message_out(PARSER, "data event received from OpenSP, incoming_data is now: " + incoming_data);
@@ -375,9 +375,9 @@ public:
   */
   void error (const ErrorEvent & event)
   {
-    string message;
-    string string_buf;
+    std::string message;
     OfxMsgType error_type = ERROR;
+    const std::string eventMessage = CharStringtostring (event.message);
 
     position = event.pos;
     message = message + "OpenSP parser: ";
@@ -396,8 +396,18 @@ public:
       error_type = ERROR;
       break;
     case SGMLApplication::ErrorEvent::otherError:
-      message = message + "otherError (misc parse error):";
-      error_type = ERROR;
+      // #60: If the SGML parser encounters a non-ascii char, it sends an error
+      // message, even though those characters are being forwarded just fine.
+      // Hence we count the occurrence of those errors and subtract it from the
+      // final number of errors.
+      if (eventMessage.find(MESSAGE_NON_SGML_CHAR) != std::string::npos) {
+        ++errorCountToIgnore;
+        message = message + "ignored character error:";
+        error_type = INFO;
+      } else {
+        message = message + "otherError (misc parse error):";
+        error_type = ERROR;
+      }
       break;
     case SGMLApplication::ErrorEvent::warning:
       message = message + "warning (Not actually an error.):";
@@ -410,7 +420,7 @@ public:
     default:
       message = message + "OpenSP sent an unknown error to LibOFX (You probably have a newer version of OpenSP):";
     }
-    message =	message + "\n" + CharStringtostring (event.message, string_buf);
+    message =	message + "\n" + eventMessage;
     message_out (error_type, message);
   }
 
@@ -443,8 +453,9 @@ int ofx_proc_sgml(LibofxContext * libofx_context, int argc, char * const* argv)
   parserKit.setOption (ParserEventGeneratorKit::showOpenEntities);
   EventGenerator *egp =	parserKit.makeEventGenerator (argc, argv);
   egp->inhibitMessages (true);	/* Error output is handled by libofx not OpenSP */
-  OFXApplication *app = new OFXApplication(libofx_context);
-  unsigned nErrors = egp->run (*app); /* Begin parsing */
+  OFXApplication app(libofx_context);
+  unsigned originalErrorCount = egp->run (app); /* Begin parsing */
+  unsigned nErrors = originalErrorCount - app.getErrorCountToIgnore(); // but ignore certain known errors that we want to ignore
   delete egp;  //Note that this is where bug is triggered
   return nErrors > 0;
 }
